"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AutoDoorsSystem = void 0;
const SystemStatus_1 = require("./SystemStatus");
const AutoOpenDoor_1 = require("../misc/AutoOpenDoor");
const events_1 = require("../events");
/**
 * Represents a system for doors that open after a period of time.
 *
 * See {@link AutoDoorsSystemEvents} for events to listen to.
 */
class AutoDoorsSystem extends SystemStatus_1.SystemStatus {
    constructor(ship, systemType, data) {
        super(ship, systemType, data);
        this.dirtyBit || (this.dirtyBit = 0);
        this.doors || (this.doors = []);
        this.doors = this.doors.map((door, i) => typeof door === "boolean"
            ? new AutoOpenDoor_1.AutoOpenDoor(this, i, door)
            : door);
    }
    Deserialize(reader, spawn) {
        if (spawn) {
            for (let i = 0; i < this.doors.length; i++) {
                const open = reader.bool();
                this.doors.push(new AutoOpenDoor_1.AutoOpenDoor(this, i, open));
            }
        }
        else {
            const mask = reader.upacked();
            for (let i = 0; i < this.doors.length; i++) {
                if (mask & (1 << i)) {
                    const isOpen = reader.bool();
                    if (isOpen) {
                        this.openDoor(i);
                    }
                    else {
                        this.closeDoor(i);
                    }
                }
            }
        }
    }
    Serialize(writer, spawn) {
        if (spawn) {
            for (let i = 0; i < this.doors.length; i++) {
                this.doors[i].Serialize(writer, spawn);
            }
        }
        else {
            writer.upacked(this.dirtyBit);
            for (let i = 0; i < this.doors.length; i++) {
                if (this.dirtyBit & (1 << i)) {
                    this.doors[i].Serialize(writer, spawn);
                }
            }
        }
        this.dirtyBit = 0;
    }
    async _openDoor(doorId, player, rpc) {
        const door = this.doors[doorId];
        if (!door)
            return;
        door.isOpen = true;
        this.dirtyBit |= 1 << doorId;
        this.dirty = true;
        const ev = await door.emit(new events_1.DoorsDoorOpenEvent(this.room, this, rpc, player, door));
        if (ev.reverted) {
            door.isOpen = false;
            return;
        }
        if (ev.alteredDoor !== door) {
            door.isOpen = false;
            ev.alteredDoor.isOpen = true;
        }
    }
    /**
     * Open a door by its ID. This is a host operation on official servers.
     * @param doorId The ID of the door to open.
     */
    async openDoor(doorId) {
        await this._openDoor(doorId, this.room.myPlayer, undefined);
    }
    async _closeDoor(doorId, player, rpc) {
        const door = this.doors[doorId];
        if (!door)
            return;
        door.isOpen = false;
        door.timer = 10;
        this.dirtyBit |= 1 << doorId;
        this.dirty = true;
        const ev = await door.emit(new events_1.DoorsDoorCloseEvent(this.room, this, rpc, player, door));
        if (ev.reverted) {
            door.isOpen = true;
            return;
        }
        if (ev.alteredDoor !== door) {
            door.isOpen = true;
            ev.alteredDoor.isOpen = false;
        }
    }
    /**
     * Close a door by its ID. This is a host operation on official servers.
     * @param doorId The ID of the door to close.
     */
    async closeDoor(doorId) {
        await this._closeDoor(doorId, this.room.myPlayer, undefined);
    }
    Detoriorate(delta) {
        for (const door of this.doors) {
            if (!door.isOpen && door.DoUpdate(delta)) {
                this.dirty = true;
            }
        }
    }
}
exports.AutoDoorsSystem = AutoDoorsSystem;
//# sourceMappingURL=AutoDoorsSystem.js.map