"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReactorSystem = void 0;
const constant_1 = require("@skeldjs/constant");
const SystemStatus_1 = require("./SystemStatus");
const events_1 = require("../events");
const endgame_1 = require("../endgame");
/**
 * Represents a system responsible for handling reactor consoles.
 *
 * See {@link ReactorSystemEvents} for events to listen to.
 */
class ReactorSystem extends SystemStatus_1.SystemStatus {
    constructor(ship, systemType, data) {
        var _a;
        super(ship, systemType, data);
        this._lastUpdate = 0;
        (_a = this.timer) !== null && _a !== void 0 ? _a : (this.timer = 10000);
        this.completed || (this.completed = new Set);
    }
    get sabotaged() {
        return this.timer < 10000;
    }
    /* eslint-disable-next-line @typescript-eslint/no-unused-vars */
    Deserialize(reader, spawn) {
        this.timer = reader.float();
        const num_consoles = reader.upacked();
        this.completed.clear();
        for (let i = 0; i < num_consoles; i++) {
            this.completed.add(reader.upacked());
        }
    }
    /* eslint-disable-next-line @typescript-eslint/no-unused-vars */
    Serialize(writer, spawn) {
        writer.float(this.timer);
        const completed = [...this.completed];
        writer.upacked(completed.length);
        for (let i = 0; i < completed.length; i++) {
            writer.upacked(completed[i]);
        }
    }
    async _addConsole(player, consoleid, rpc) {
        this.completed.add(consoleid);
        this.dirty = true;
        const ev = await this.emit(new events_1.ReactorConsoleAddEvent(this.room, this, undefined, player, consoleid));
        if (ev.reverted) {
            this.completed.delete(ev.consoleId);
            return;
        }
        if (ev.alteredConsoleId !== consoleid) {
            this.completed.delete(consoleid);
            this.completed.add(ev.alteredConsoleId);
        }
        if (this.completed.size >= 2) {
            await this._repair(player, rpc);
        }
    }
    /**
     * Add a completed console. (Same as a player placing their hand on a
     * console)
     * @param consoleid The ID of the console to add.
     */
    async addConsole(consoleid) {
        if (this.room.hostIsMe) {
            await this._addConsole(this.room.myPlayer, consoleid, undefined);
        }
        else {
            await this._sendRepair(0x40 | consoleid);
        }
    }
    async _removeConsole(player, consoleid, rpc) {
        this.completed.delete(consoleid);
        this.dirty = true;
        const ev = await this.emit(new events_1.ReactorConsoleRemoveEvent(this.room, this, undefined, player, consoleid));
        if (ev.reverted) {
            return this.completed.add(ev.consoleId);
        }
        if (ev.alteredConsoleId !== consoleid) {
            this.completed.add(consoleid);
            this.completed.delete(ev.alteredConsoleId);
        }
    }
    /**
     * Remove a completed console. (Same as a player removing their hand from a
     * console)
     * @param consoleid The ID of the console to add.
     */
    async removeConsole(consoleid) {
        if (this.room.hostIsMe) {
            await this._removeConsole(this.room.myPlayer, consoleid, undefined);
        }
        else {
            await this._sendRepair(0x20 | consoleid);
        }
    }
    async HandleSabotage(player, rpc) {
        this.timer = 45;
        const oldCompleted = this.completed;
        this.completed = new Set;
        const ev = await this.emit(new events_1.SystemSabotageEvent(this.room, this, rpc, player));
        if (ev.reverted) {
            this.timer = 10000;
            this.completed = oldCompleted;
        }
    }
    async _repair(player, rpc) {
        const oldTimer = this.timer;
        const oldCompleted = this.completed;
        this.timer = 10000;
        this.completed = new Set;
        this.dirty = true;
        const ev = await this.emit(new events_1.ReactorConsolesResetEvent(this.room, this, rpc, player));
        if (ev.reverted) {
            this.timer = oldTimer;
            this.completed = oldCompleted;
        }
    }
    async repair() {
        if (this.room.hostIsMe) {
            await this._repair(this.room.myPlayer, undefined);
        }
        else {
            await this._sendRepair(0x10);
        }
    }
    /* eslint-disable-next-line @typescript-eslint/no-unused-vars */
    async HandleRepair(player, amount, rpc) {
        const consoleId = amount & 0x3;
        if (amount & 0x40) {
            await this._addConsole(player, consoleId, rpc);
        }
        else if (amount & 0x20) {
            await this._removeConsole(player, consoleId, rpc);
        }
        else if (amount & 0x10) {
            await this._repair(player, rpc);
        }
        this.dirty = true;
    }
    Detoriorate(delta) {
        if (!this.sabotaged)
            return;
        this.timer -= delta;
        this._lastUpdate += delta;
        if (this._lastUpdate > 2) {
            this._lastUpdate = 0;
            this.dirty = true;
        }
        if (this.timer <= 0) {
            this.room.registerEndGameIntent(new endgame_1.EndGameIntent(endgame_1.AmongUsEndGames.ReactorSabotage, constant_1.GameOverReason.ImpostorBySabotage, {}));
        }
    }
}
exports.ReactorSystem = ReactorSystem;
//# sourceMappingURL=ReactorSystem.js.map